/* 
	Team 26: Tod Reynolds, Pill Park, Joshua Peng
	Lab5: PIC32MX: Interfacing with Force Sensors from a Scale 
*/

/** INCLUDES ***************************************************/
#include "HardwareProfile.h"

/** Constants **************************************************/

#define TRUE 		1
#define FALSE		0

#define DESIRED_BAUDRATE    	(19200)      // The desired BaudRate 
#define NUM_DATA_POINTS 4					 // Number of data points being sent to PC, 4 input sensors

/** Function Declarations **************************************/

void initUART2(int pbClk); 		//Initialize RS232

void sendDataRS232(void);		//Function to send RS232

void initADC();					//Initializes ADC

//void initInterruptController(void); 

/** Global Variables *******************************************/

char RS232_Out_Buffer[32];

short int top_left;
short int top_right;
short int bottom_left;
short int bottom_right;
volatile unsigned char data;


short int top_left_initial;
short int top_right_initial;
short int bottom_left_initial;
short int bottom_right_initial;


/** Main Function **********************************************/

int main(void)
{
	int	pbClk;

	// This line of code only needs to be used if your pins are Analog Input (B port)
    AD1PCFG = 0xFFFF;
		
	// Configure the proper PB frequency and the number of wait states
	pbClk = SYSTEMConfigPerformance(SYS_FREQ);
		
	// Allow vector interrupts
	INTEnableSystemMultiVectoredInt();
	//initInterruptController();
	mInitAllLEDs();
	initUART2(pbClk);
	initADC();
	
	while(1)
	{
/*
// read in each ADC pin
	top_left= ReadADC10(0);
	top_right= ReadADC10(1);
	bottom_left= ReadADC10(2);
	bottom_right= ReadADC10(3);*/

	top_left= ReadADC10(0)-top_left_initial;
	top_right= ReadADC10(1)-top_right_initial;
	bottom_left= ReadADC10(2)-bottom_left_initial;
	bottom_right= ReadADC10(3)-bottom_right_initial;


    if (top_left != 0) {
	mLED_0_On();
	}
	else if (top_left == 0) {
	mLED_0_Off();
	}	
	if (top_right != 0){
	mLED_1_On();
 	} 
	else if (top_right == 0) {
	mLED_1_Off();
	}
	if(bottom_left != 0){
	mLED_2_On();
	}
	else if (bottom_left == 0) {
	mLED_2_Off();
	}
	if(bottom_right != 0){
	mLED_3_On();
	}
	else if (bottom_right == 0) {
	mLED_3_Off();
	}

//Send to PC via RS232
	sendDataRS232();
	}

} //end main


/** Interrupt Handlers *****************************************/
/*
void __ISR( _TIMER_3_VECTOR, ipl7) T3Interrupt( void)
{

	top_left= ReadADC10(0)-top_left_initial;
	top_right= ReadADC10(1)-top_right_initial;
	bottom_left= ReadADC10(2)-bottom_left_initial;
	bottom_right= ReadADC10(3)-bottom_right_initial;
	
	// clear interrupt flag and exit
	mT3ClearIntFlag();
} // T3 Interrupt
*/
void __ISR(_UART2_VECTOR, ipl2) IntUart2Handler(void)
{
	char data;

	// Is this an RX interrupt?
	if(mU2RXGetIntFlag())
	{
		// Clear the RX interrupt Flag
	    mU2RXClearIntFlag();
	
		data = ReadUART2();
		// Echo what we just received.
		//putcUART2(data);
		
		switch(data)
		{

			case 'p':
				top_left_initial=ReadADC10(0);
				top_right_initial=ReadADC10(1);
				bottom_left_initial=ReadADC10(2);
				bottom_right_initial=ReadADC10(3);
				break;

		}
				
 
		// Toggle LED to indicate UART activity
		mLED_0_Toggle();
 
	}

	// We don't care about TX interrupt
	if ( mU2TXGetIntFlag() )
	{
		mU2TXClearIntFlag();
	}
}


/** Other Functions ********************************************/

//Funtion to Initialize serial port
void initUART2(int pbClk)
{
	 // define setup Configuration 1 for OpenUARTx
 		// Module Enable 
 		// Work in IDLE mode 
 		// Communication through usual pins 
 		// Disable wake-up 
 		// Loop back disabled 
 		// Input to Capture module from ICx pin 
 		// no parity 8 bit 
 		// 1 stop bit 
 		// IRDA encoder and decoder disabled 
 		// CTS and RTS pins are disabled 
 		// UxRX idle state is '1' 
 		// 16x baud clock - normal speed
 	#define config1 	UART_EN | UART_IDLE_CON | UART_RX_TX | UART_DIS_WAKE | UART_DIS_LOOPBACK | UART_DIS_ABAUD | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_IRDA_DIS | UART_DIS_BCLK_CTS_RTS| UART_NORMAL_RX | UART_BRGH_SIXTEEN
	 
	 // define setup Configuration 2 for OpenUARTx
	 	// IrDA encoded UxTX idle state is '0'
	 	// Enable UxRX pin
	 	// Enable UxTX pin
	 	// Interrupt on transfer of every character to TSR 
	 	// Interrupt on every char received
	 	// Disable 9-bit address detect
	 	// Rx Buffer Over run status bit clear
	 #define config2		UART_TX_PIN_LOW | UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR | UART_ADR_DETECT_DIS | UART_RX_OVERRUN_CLEAR	
 
 	// Open UART2 with config1 and config2
 	OpenUART2( config1, config2, pbClk/16/DESIRED_BAUDRATE-1);	// calculate actual BAUD generate value.
 		
 	// Configure UART2 RX Interrupt with priority 2
 	ConfigIntUART2(UART_INT_PR2 | UART_RX_INT_EN);
}

//Funtion initialization to send RS232
void sendDataRS232()
{
	
	sprintf(RS232_Out_Buffer,"%d %d %d %d\n",top_left, top_right, bottom_left, bottom_right);
	putsUART2(RS232_Out_Buffer);
}


//Function to initialize ADC
void initADC()
{  
// Configure the proper PB frequency and the number of wait states
	SYSTEMConfigPerformance(SYS_FREQ);

	// configure and enable the ADC
	CloseADC10();	// ensure the ADC is off before setting the configuration

	// define setup parameters for OpenADC10
				// Turn module on | output in integer | trigger mode auto | enable  autosample
	#define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

	// define setup parameters for OpenADC10
			    // ADC ref external    | disable offset test    | enable scan mode | perform 4 samples | use one buffer | use MUXA mode
	#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_4 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

	// define setup parameters for OpenADC10
	// 				  use ADC internal clock | set sample time
	#define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

	// define setup parameters for OpenADC10
				// set AN1 thru AN4
	#define PARAM4	ENABLE_AN1_ANA | ENABLE_AN2_ANA | ENABLE_AN3_ANA | ENABLE_AN4_ANA

	// define setup parameters for OpenADC10
	// do not assign channels to scan
	#define PARAM5	SKIP_SCAN_AN0 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

	// use ground as neg ref for A 
	SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF); 
	OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above

	EnableADC10(); // Enable the ADC

	while ( ! mAD1GetIntFlag() ) { } // wait for the first conversion to complete so there will be valid data in ADC result registers
}	

/*
void initInterruptController(void)
{
	// init Timer3 mode and period (PR3) // produces 1ms period
	//OpenTimer3( T3_ON | T3_PS_1_2 | T3_SOURCE_INT, 0x9C3F);
	
	mT3SetIntPriority( 7); 	// set Timer3 Interrupt Priority
	mT3ClearIntFlag(); 		// clear interrupt flag
	mT3IntEnable( 1);		// enable timer3 interrupts
}*/

